/* -------------------------------------------------------------------------*
 * Samples for the Attolic True Studio and the GCC                          *
 * Copyright (C) Lucjan Bryndza <lucjan.bryndza@ep.com.pl>  19.11.2010      *
 * Example No3 for the STM32 Butterfly2 and KamodLED8                       *
 * Joystick state on leds.          KamodLED8 D0-D7 lines should be         *
 * connected to the PE0-PE7 on the STM32Butterfly board                     *
 * http://www.boff.pl                                                       *
 * -------------------------------------------------------------------------*/

#include "stm32f10x_lib.h"
#include "system.h"
#include <stdbool.h>

#define LEDS_PORT GPIOE
#define LEDS_BITS 0xff
#define LED_PORT_APB2 RCC_APB2Periph_GPIOE

#define DELAY_LOOP_CYCLES 500000


/* ----------------------------------------------------------------------- */
//Joy configuration for the STM32 butterfly 1 and 2
#ifdef STM32_BUTTERFLY_1 /* Butterfly 1 */

#define JOY_PORT GPIOE
#define JOY_PORT_APB2 RCC_APB2Periph_GPIOE
#define JOY_PIN_L 12
#define JOY_PIN_R 11
#define JOY_PIN_D 10
#define JOY_PIN_U 9
#define JOY_PIN_OK 8

#else /* Butterfly 2 */

#define JOY_PORT GPIOC
#define JOY_PORT_APB2 RCC_APB2Periph_GPIOC
#define JOY_PIN_L 11
#define JOY_PIN_R 10
#define JOY_PIN_D 9
#define JOY_PIN_U 8
#define JOY_PIN_OK 7

#endif /*STM32_BUTTERFLY_1*/

#define JOY_BITS ((1<<JOY_PIN_L) | (1<<JOY_PIN_R) | (1<<JOY_PIN_D) | (1<<JOY_PIN_U) | (1<<JOY_PIN_OK))

/* ----------------------------------------------------------------------- */

enum keys
{
   JOY_OK = 0x01,
   JOY_U =  0x02,
   JOY_D =  0x04,
   JOY_R =  0x08,
   JOY_L =  0x10
};

//Get joy keystate
static inline int joy_keys(void)
{
    return (~io_get_mask(JOY_PORT, JOY_BITS) & JOY_BITS) >> JOY_PIN_OK;
}


/* ----------------------------------------------------------------------- */
int main(void)
{
    //LED as output
    RCC->APB2ENR |= LED_PORT_APB2; //Enable PE in APB2
    io_config_ext(LEDS_PORT,LEDS_BITS,GPIO_MODE_10MHZ,GPIO_CNF_GPIO_PP);
    
   //JOY pin configuration input pullup VCC
    RCC->APB2ENR |= JOY_PORT_APB2;
    io_config_ext(JOY_PORT, JOY_BITS, GPIO_MODE_INPUT, GPIO_CNF_IN_PULLUP );
    io_set_clr_mask( JOY_PORT, JOY_BITS, JOY_BITS );
    
    int leds_on = 0;
    //main loop
    while(true)
    {
	int jkey = joy_keys();
	if(jkey & JOY_OK )
	    leds_on = 0;
	else
	    leds_on |= jkey>>1;        
	io_set_clr_mask(LEDS_PORT, leds_on, LEDS_BITS );
    }
    return 0;
}
/* ----------------------------------------------------------------------- */

